#version 430 core
/*
* Copyright 2018 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

#define MAX_LIGHTS 128

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

out vec4 color;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform sampler2D diffuseTexture;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

const float levels = 5.0;

uniform int pointLCnt;
uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} 
pointL[MAX_LIGHTS];

vec3 phong(vec3 n, vec3 direction, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(direction);
	direction = normalize(direction);
	float att = 1.0;	
	if(attenuate){ 
		att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	}
	vec3 ref = reflect(-direction, n);
	float brightness = max(0, dot(n, direction));
	float level = floor(brightness * levels);
	brightness = level / levels;
	return (diffuseF * diffuseC * brightness + specularF * specularC * pow(max(0, dot(ref, v)), alpha)) * att; 
	}

void main() {	

	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);

	vec3 texColor = texture(diffuseTexture, vert.uv).rgb;

	color = vec4(texColor * materialCoefficients.x, 1); // ambient
			
		
	// add directional light contribution
	color.rgb += phong(n, dirL.direction - vert.position_world, v, dirL.color * texColor, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0));
		


    for(int i = 0; i < pointLCnt; i++) 
	{
		// add point light contribution
		color.rgb += phong(n, pointL[i].position - vert.position_world, v, pointL[i].color * texColor, materialCoefficients.y, pointL[i].color, materialCoefficients.z, specularAlpha, true, pointL[i].attenuation);
    }
	


}